@@ -44,7 +44,8 @@ from electrum import WalletStorage
 # from electrum.synchronizer import Synchronizer
 # from electrum.verifier import SPV
 # from electrum.util import DebugMem
-from electrum.util import UserCancelled, print_error
+from electrum.util import (UserCancelled, print_error,
+                           WalletFileException, BitcoinException)
 # from electrum.wallet import Abstract_Wallet
 
 from .installwizard import InstallWizard, GoBack
@@ -185,42 +186,51 @@ class ElectrumGui:
 
     def start_new_window(self, path, uri):
         '''Raises the window for the wallet if it is open.  Otherwise
-        opens the wallet and creates a new window for it.'''
-        for w in self.windows:
-            if w.wallet.storage.path == path:
-                w.bring_to_top()
-                break
-        else:
+        opens the wallet and creates a new window for it'''
+        try:
+            wallet = self.daemon.load_wallet(path, None)
+        except BaseException as e:
+            traceback.print_exc(file=sys.stdout)
+            d = QMessageBox(QMessageBox.Warning, _('Error'),
+                            _('Cannot load wallet') + ' (1):\n' + str(e))
+            d.exec_()
+            return
+        if not wallet:
+            storage = WalletStorage(path, manual_upgrades=True)
+            wizard = InstallWizard(self.config, self.app, self.plugins, storage)
             try:
-                wallet = self.daemon.load_wallet(path, None)
-            except BaseException as e:
-                traceback.print_exc(file=sys.stdout)
+                wallet = wizard.run_and_get_wallet(self.daemon.get_wallet)
+            except UserCancelled:
+                pass
+            except GoBack as e:
+                print_error('[start_new_window] Exception caught (GoBack)', e)
+            except (WalletFileException, BitcoinException) as e:
+                traceback.print_exc(file=sys.stderr)
                 d = QMessageBox(QMessageBox.Warning, _('Error'),
-                                _('Cannot load wallet:') + '\n' + str(e))
+                                _('Cannot load wallet') + ' (2):\n' + str(e))
                 d.exec_()
                 return
-            if not wallet:
-                storage = WalletStorage(path, manual_upgrades=True)
-                wizard = InstallWizard(self.config, self.app, self.plugins, storage)
-                try:
-                    wallet = wizard.run_and_get_wallet()
-                except UserCancelled:
-                    pass
-                except GoBack as e:
-                    print_error('[start_new_window] Exception caught (GoBack)', e)
+            finally:
                 wizard.terminate()
-                if not wallet:
-                    return
+            if not wallet:
+                return
+
+            if not self.daemon.get_wallet(wallet.storage.path):
+                # wallet was not in memory
                 wallet.start_threads(self.daemon.network)
                 self.daemon.add_wallet(wallet)
-            try:
-                w = self.create_window_for_wallet(wallet)
-            except BaseException as e:
-                traceback.print_exc(file=sys.stdout)
-                d = QMessageBox(QMessageBox.Warning, _('Error'),
-                                _('Cannot create window for wallet:') + '\n' + str(e))
-                d.exec_()
-                return
+        try:
+            for w in self.windows:
+                if w.wallet.storage.path == wallet.storage.path:
+                    w.bring_to_top()
+                    return
+            w = self.create_window_for_wallet(wallet)
+        except BaseException as e:
+            traceback.print_exc(file=sys.stdout)
+            d = QMessageBox(QMessageBox.Warning, _('Error'),
+                            _('Cannot create window for wallet') + ':\n' + str(e))
+            d.exec_()
+            return
         if uri:
             w.pay_to_URI(uri)
         w.bring_to_top()
