@@ -65,6 +65,7 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
         self.end_timestamp = None
         self.years = []
         self.create_toolbar_buttons()
+        self.wallet = None
 
     def format_date(self, d):
         return str(datetime.date(d.year, d.month, d.day)) if d else _('None')
@@ -235,7 +236,7 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
             label = tx_item['label']
             status, status_str = self.wallet.get_tx_status(tx_hash, height, conf, timestamp)
             has_invoice = self.wallet.invoices.paid.get(tx_hash)
-            icon = QIcon(":icons/" + TX_ICONS[status])
+            icon = self.icon_cache.get(":icons/" + TX_ICONS[status])
             v_str = self.parent.format_amount(value, True, whitespaces=True)
             balance_str = self.parent.format_amount(balance, whitespaces=True)
             entry = ['', tx_hash, status_str, label, v_str, balance_str]
@@ -253,7 +254,7 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
             item.setToolTip(0, str(conf) + " confirmation" + ("s" if conf != 1 else ""))
             item.setData(0, SortableTreeWidgetItem.DataRole, (status, conf))
             if has_invoice:
-                item.setIcon(3, QIcon(":icons/seal"))
+                item.setIcon(3, self.icon_cache.get(":icons/seal"))
             for i in range(len(entry)):
                 if i>3:
                     item.setTextAlignment(i, Qt.AlignRight | Qt.AlignVCenter)
@@ -301,8 +302,10 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
             item.setText(3, label)
 
     def update_item(self, tx_hash, height, conf, timestamp):
+        if self.wallet is None:
+            return
         status, status_str = self.wallet.get_tx_status(tx_hash, height, conf, timestamp)
-        icon = QIcon(":icons/" +  TX_ICONS[status])
+        icon = self.icon_cache.get(":icons/" +  TX_ICONS[status])
         items = self.findItems(tx_hash, Qt.UserRole|Qt.MatchContains|Qt.MatchRecursive, column=1)
         if items:
             item = items[0]
@@ -336,7 +339,8 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
             menu.addAction(_("Remove"), lambda: self.remove_local_tx(tx_hash))
         menu.addAction(_("Copy {}").format(column_title), lambda: self.parent.app.clipboard().setText(column_data))
         for c in self.editable_columns:
-            menu.addAction(_("Edit {}").format(self.headerItem().text(c)), lambda: self.editItem(item, c))
+            menu.addAction(_("Edit {}").format(self.headerItem().text(c)),
+                           lambda bound_c=c: self.editItem(item, bound_c))
         menu.addAction(_("Details"), lambda: self.parent.show_transaction(tx))
         if is_unconfirmed and tx:
             rbf = is_mine and not tx.is_final()
@@ -347,7 +351,7 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
                 if child_tx:
                     menu.addAction(_("Child pays for parent"), lambda: self.parent.cpfp(tx, child_tx))
         if pr_key:
-            menu.addAction(QIcon(":icons/seal"), _("View invoice"), lambda: self.parent.show_invoice(pr_key))
+            menu.addAction(self.icon_cache.get(":icons/seal"), _("View invoice"), lambda: self.parent.show_invoice(pr_key))
         if tx_URL:
             menu.addAction(_("View on block explorer"), lambda: webbrowser.open(tx_URL))
         menu.exec_(self.viewport().mapToGlobal(position))
@@ -411,7 +415,7 @@ class HistoryList(MyTreeWidget, AcceptFileDragDrop):
                 lines.append([item['txid'], item.get('label', ''), item['confirmations'], item['value'], item['date']])
             else:
                 lines.append(item)
-        with open(fileName, "w+") as f:
+        with open(fileName, "w+", encoding='utf-8') as f:
             if is_csv:
                 import csv
                 transaction = csv.writer(f, lineterminator='\n')
