@@ -1,5 +1,3 @@
-import threading
-
 from binascii import hexlify, unhexlify
 
 from electrum.util import bfh, bh2u
@@ -9,7 +7,7 @@ from electrum.bitcoin import (b58_address_to_hash160, xpub_from_pubkey,
 from electrum import constants
 from electrum.i18n import _
 from electrum.plugins import BasePlugin
-from electrum.transaction import deserialize
+from electrum.transaction import deserialize, Transaction
 from electrum.keystore import Hardware_KeyStore, is_xpubkey, parse_xpubkey
 from electrum.base_wizard import ScriptTypeNotSupported
 
@@ -50,6 +48,8 @@ class KeepKeyCompatibleKeyStore(Hardware_KeyStore):
         for txin in tx.inputs():
             pubkeys, x_pubkeys = tx.get_sorted_pubkeys(txin)
             tx_hash = txin['prevout_hash']
+            if txin.get('prev_tx') is None and not Transaction.is_segwit_input(txin):
+                raise Exception(_('Offline signing with {} is not supported for legacy inputs.').format(self.device))
             prev_tx[tx_hash] = txin['prev_tx']
             for x_pubkey in x_pubkeys:
                 if not is_xpubkey(x_pubkey):
@@ -72,8 +72,6 @@ class KeepKeyCompatiblePlugin(HW_PluginBase):
 
     def __init__(self, parent, config, name):
         HW_PluginBase.__init__(self, parent, config, name)
-        self.main_thread = threading.current_thread()
-        # FIXME: move to base class when Ledger is fixed
         if self.libraries_available:
             self.device_manager().register_devices(self.DEVICE_IDS)
 
@@ -101,7 +99,7 @@ class KeepKeyCompatiblePlugin(HW_PluginBase):
             return None
 
     def create_client(self, device, handler):
-        # disable bridge because it seems to never returns if keepkey is plugged
+        # disable bridge because it seems to never returns if KeepKey is plugged
         #transport = self._try_bridge(device) or self._try_hid(device)
         transport = self._try_hid(device)
         if not transport:
@@ -346,7 +344,7 @@ class KeepKeyCompatiblePlugin(HW_PluginBase):
                     elif addrtype == constants.net.ADDRTYPE_P2SH:
                         txoutputtype.script_type = self.types.PAYTOSCRIPTHASH
                     else:
-                        raise BaseException('addrtype: ' + str(addrtype))
+                        raise Exception('addrtype: ' + str(addrtype))
                 txoutputtype.address = address
             return txoutputtype
 
@@ -397,7 +395,7 @@ class KeepKeyCompatiblePlugin(HW_PluginBase):
             o.script_pubkey = bfh(vout['scriptPubKey'])
         return t
 
-    # This function is called from the trezor libraries (via tx_api)
+    # This function is called from the TREZOR libraries (via tx_api)
     def get_tx(self, tx_hash):
         tx = self.prev_tx[tx_hash]
         return self.electrum_tx_to_txtype(tx)
