@@ -72,10 +72,10 @@ class LabelsPlugin(BasePlugin):
             kwargs['headers']['Content-Type'] = 'application/json'
         response = requests.request(method, url, **kwargs)
         if response.status_code != 200:
-            raise BaseException(response.status_code, response.text)
+            raise Exception(response.status_code, response.text)
         response = response.json()
         if "error" in response:
-            raise BaseException(response["error"])
+            raise Exception(response["error"])
         return response
 
     def do_request_safe(self, *args, **kwargs):
